'use strict';

const schema = require('./schema.js');
const pkg = require('../package.json');
const index = require('./generate/index.js');

/**
 * @module index
 */
const initialized = Symbol(pkg.name);
const loader = function loader(content) {
  if (this.cacheable) {
    this.cacheable(true);
  }
  const { _compiler: compiler } = this;
  const logger = this.getLogger(pkg.name);
  const options = this.getOptions(schema);
  index.generate(`${this.resourcePath}.d.ts`, content, options).catch(error => {
    logger.error(error.message);
  });
  if (compiler) {
    if (!compiler[initialized]) {
      compiler[initialized] = true;
      if (compiler.watchMode) {
        compiler.hooks.watchClose.tap(pkg.name, () => index.terminate());
      } else {
        compiler.hooks.done.tapPromise(pkg.name, () => index.terminate());
      }
    }
  }
  return content;
};

module.exports = loader;
